/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.apache.openejb.config.rules;

import org.apache.openejb.config.EjbModule;
import org.apache.openejb.jee.EnterpriseBean;
import org.apache.openejb.jee.ResourceEnvRef;
import org.apache.openejb.jee.ResourceRef;
import static org.apache.openejb.jee.TransactionType.CONTAINER;

import java.util.Collection;

/**
 * Excerpt from EJB 3.0 Core spec, chapter <b>16.12. UserTransaction Interface</b>:
 * <p>
 * <i>The container must make the UserTransaction interface available to the enterprise beans that are
 * allowed to use this interface (only session and message-driven beans with bean-managed transaction
 * demarcation are allowed to use this interface)</i>
 * </p>
 *
 * @version $Rev: 778348 $ $Date: 2009-05-25 01:22:37 -0700 (Mon, 25 May 2009) $
 */
public class CheckUserTransactionRefs extends ValidationBase {

    public void validate(EjbModule ejbModule) {
        for (EnterpriseBean bean : ejbModule.getEjbJar().getEnterpriseBeans()) {
            if (bean.getTransactionType() == CONTAINER) {
                Collection<ResourceRef> resRefs = bean.getResourceRef();
                for (ResourceRef resRef : resRefs) {
                    if ("javax.transaction.UserTransaction".equals(resRef.getResType())) {
                        error(bean, "userResourceRef.forbiddenForCmtdBeans", resRef.getName());
                    }
                }
            }
        }
    }
}
